use strict;
use warnings;

sub get_pixel_size {

    # Purpose: extract the pixel size from the second line of a file
    # containing particle orientation and origin information
    #
    # Input arguments:
    #   $filename - name of data file
    #
    # Return values:
    #   $pixel_size - pixel size

    my ($filename) = @_;
    open(DAT, $filename) || 
	die "Unable to open file $filename in subroutine get_pixel_size";

    <DAT>;
    my $line = <DAT>;
    my ($pixel_size) = split(/,/, $line);
    $pixel_size =~ s/\s+//g;

    close(DAT);
    return $pixel_size;
}

sub get_pif_name {

    # Purpose: get the name of the PIF file from the first line of a
    # file containing particle orientation and origin information
    #
    # Input arguments:
    #   $filename - name of data file
    #
    # Return values:
    #   $pif_file - name of PIF file

    my ($filename) = @_;
    open(DAT, $filename) || 
	die "Unable to open file $filename in subroutine get_pif_name";

    my $pif_name = <DAT>;
    $pif_name =~ s/\s+//g;

    close(DAT);
    return $pif_name;
}

sub get_boxrad {
    
    # Purpose: get the box size (radius) from a PIF file
    #
    # Input arguments:
    #   $filename - name of PIF file
    #
    # Return values:
    #   $boxrad - box size (radius)
    #
    # Note - the variable $boxrad corresponds to the field packRadius
    # in the PIF format specification.

    my ($filename) = @_;

    open(FH, $filename) || die 
	"Unable to open file $filename in subroutine get_boxrad\n";
    binmode FH;

    # Determine the endianness of the machine that created this file
    # based on the 4 bytes offset 28 bytes from the start of the
    # file. According to the PIF standard, a value of zero corresponds
    # to little endian, while a value of 1 corresponds to big
    # endian. Since we don't know the endianness of the file ahead of
    # time, the 4 bytes will arbitrarily interpreted as a big-endian
    # long int and we'll test whether or not the result is non-zero

    my ($bigend, $mode, $boxrad);
    seek(FH, 28, 0);
    read FH, $bigend, 4;
    $bigend = unpack "N", $bigend; # # Big endian interpretation

    # Determine the mode for the EM data type based on the 4 bytes
    # offset 80 bytes from the start of the file. Intepret as big- or
    # little-endian 32-bit int based on endianness of file.

    seek(FH, 80, 0);
    read FH, $mode, 4;
    if($bigend) {
	$mode = unpack "N", $mode; # Big endian interpretation
    } else {
	$mode = unpack "V", $mode; # Little endian interpretation
    }

    # If the mode is equal to 97, then we have depthcued data. In this
    # case, box radius (packRadius) is offset 2068 bytes from start of
    # file. Otherwise, the offset is 532 bytes.

    if($mode == 97) {
	seek(FH, 2068, 0);
	read FH, $boxrad, 4;
    } else {
	seek(FH, 532, 0);
	read FH, $boxrad, 4;
    }    
    
    # Interpret as big- or little-endian based on endianness of file

    if($bigend) {
	$boxrad = unpack "N", $boxrad;
    } else {
	$boxrad = unpack "V", $boxrad;
    }

    close(FH);
    return($boxrad);
}
1;
